/*
 * TreeSolverInterface.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on May 1, 2007, 7:47 PM
 *
 */

package info.frantzen.testing.ssmsimulator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Hashtable;
import java.util.Set;
import java.util.logging.Logger;

/**
 *
 * @author Lars Frantzen
 * @version 0.1
 */
public class TreeSolverInterface {
    
    private Socket soc;
    private PrintWriter out = null;
    private BufferedReader in = null;
    
    
    /**
     * When the ST_PosInt type generates a random integer value, this random value is constained by this
     * upper bound. It relates to the fd_max_integer of the Prolog solver. Still, this does
     * not prevent overflows since these integers may be added, multiplied, etc... Unfortunately
     * GNU Prolog does not do an overflow check.
     */
    public final static int MAXINTRANDOM = 10000;
    
    /** Creates a new instance of TreeSolverInterface */
    public TreeSolverInterface(Socket solverSocket, Logger logger) throws IOException {
        soc = solverSocket;
        //new Socket("localhost", solverPort);
        //soc.configureBlocking(false);
        //logger.info("treeSolver is bound to local port " + soc.getLocalPort());
        
        out = new PrintWriter(soc.getOutputStream(), true);
        in = new BufferedReader(new InputStreamReader(soc.getInputStream()));
        
        //String welcomeMessage = in.readLine();
        //logger.info("Server sends: " + welcomeMessage);
        
        // out.println("[[A,B,C], tree( node_d(op_and, node_d(op_and, node_d(op_eq, node_s(A, empty), node_s(24,empty)), node_d(op_leq, node_s(B, empty), node_s(44, empty))), node_d(op_eq, node_s(C, empty), node_d(op_mult, node_s(A,empty), node_s(B,empty)))))].");
        // System.out.println("Server sends: " + in.readLine());
        
    }
        
    public Hashtable<String,Integer> askSolver(int prologVarCounter, String query) throws IOException {
        StringBuffer sb = new StringBuffer();
        sb.append("[[X0");
        for (int i = 1; i < prologVarCounter; i++) {
            sb.append(",X");
            sb.append(Integer.toString(i));
        }
        sb.append("],");
        sb.append(query);
        sb.append("].");
        
        out.println(sb.toString());
        String solutionString = in.readLine();
        
        // If the solver reports no solution, we return null
        if (solutionString.equals("false"))
            return null;
        
        Hashtable<String,Integer> solution = new Hashtable<String,Integer>();
        int index = 1;
        int nextComma;
        for (int i = 0; i < prologVarCounter; i++) {
            nextComma = solutionString.indexOf(',', index);
            if (nextComma == -1)
                nextComma = solutionString.indexOf(']', index);
            String intString = solutionString.substring(index,nextComma);
            index = nextComma + 1;
            String key = 'X' + Integer.toString(i);
            solution.put(key, Integer.parseInt(intString));
        }
        
        return solution;
    }
    
    public Hashtable askSolverAlwaysNew(int prologVarCounter, String query, Set<Hashtable<String, Integer>> foundSolutions) throws IOException {
        if (foundSolutions.size() == 0)
            return askSolver(prologVarCounter, query);
        
        String newQuery = new String();
        String extraConstraint = new String();
        
        Object[] solutionArray = foundSolutions.toArray();
        newQuery = generateSetSolutionConstraints(solutionArray, solutionArray.length);
        String finalQuery = "tree(node_d(op_and," + newQuery + ',' + query.substring(query.indexOf('(')) + "))";
        return askSolver(prologVarCounter, finalQuery);
    }
    
    private String generateSetSolutionConstraints(Object[] solutions, int numberSolutions) {
        if (numberSolutions == 1) {
            return generateSolutionConstraints((Hashtable<String, Integer>) ((Hashtable<String, Integer>) solutions[0]).clone());
        } else { // currentSolution.size() > 1
            int solutionIndex = numberSolutions-1;
            String leftString = generateSolutionConstraints((Hashtable<String, Integer>) ((Hashtable<String, Integer>) solutions[solutionIndex]).clone());
            return "node_d(op_and," + leftString + ',' + generateSetSolutionConstraints(solutions, numberSolutions-1) + ')';
        }
    }
    
    private String generateSolutionConstraints(Hashtable<String,Integer> currentSolution) {
        int size = currentSolution.size();
        if (size == 1) {
            return generateSingleVarSolutionConstraint(0,currentSolution.get("X0"));
        } else { // currentSolution.size() > 1
            int varIndex = size-1;
            String leftString = generateSingleVarSolutionConstraint(varIndex, currentSolution.get("X"+varIndex));
            currentSolution.remove("X"+varIndex);
            return "node_d(op_or," + leftString + ',' + generateSolutionConstraints(currentSolution) + ')';
        }
    }
    
    private String generateSingleVarSolutionConstraint(int varnumber, int varvalue) {
        return "node_d(op_neq,node_s(X" + varnumber + ",empty),node_s(" + varvalue + ",empty))";
    }
    
}
