/* Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package info.frantzen.testing.ssmsimulator.ssm;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

/**
 * A location is a symbolic variant of a state. Locations are connected via
 * {@link Switch switches}. Every location maintains its own set of outgoing switches.
 * @version 0.1
 * @author Lars Frantzen
 */
public class Location {
    
    private String name;
    private HashSet<Switch> outgoingSwitches;
    private int visitedCount = 0;
    
    /**
     * Creates a new Location.
     * @param name the name of the Location
     * @param outgoingSwitches the set of outgoing Switches
     */
    public Location(String name, HashSet outgoingSwitches) {
        this.name = name;
        this.outgoingSwitches = outgoingSwitches;
    }
    
    /**
     * Creates a new Location with an empty set of outgoing Switches.
     * @param name the name of the Location
     */
    public Location(String name) {
        this.name = name;
        this.outgoingSwitches = new HashSet<Switch>();
    }
    
    public void visited() {
        visitedCount++;
    }
    
    public int getVistedCount() {
        return visitedCount;
    }
    
    /**
     * Adds an outgoing Switch to a Location.
     * @param s the switch to be added
     */
    public void addOutgoingSwitch(Switch s) {
        outgoingSwitches.add(s);
    }
    
    /**
     * Getter for the name attribute.
     * @return the name attribute
     */
    public String getName() {
        return name;
    }
    
    /**
     * Setter for the name attribute.
     * @param name the new name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Getter for the set of outgoing Switches.
     * @return the set of outgoing Switches
     */
    public HashSet<Switch> getOutgoingSwitches() {
        return outgoingSwitches;
    }
    
    /**
     * Setter for the set of outgoing Switches.
     * @param outgoingSwitches the new set of outgoing Switches
     */
    public void setOutgoingSwitches(HashSet<Switch> outgoingSwitches) {
        this.outgoingSwitches = outgoingSwitches;
    }
    
    /**
     * A String representation of this Location.
     * @return a string representation of this Location
     */
    public String toString() {
        return name;
    }
    
    /**
     * Checks the consistency, i.e. no outgoing Switch is null and the name
     * is not the emty String.
     * @return true if the consistency is ok, false otherwise
     */
    public boolean checkConsistency() {
        for (Iterator it = outgoingSwitches.iterator() ; it.hasNext() ;) {
            if ((Switch) it.next() == null)
                return false;
        }
        return (name != "");
    }

    public Collection<Location> traverseDotGraph(StringBuffer sb) {
        Collection<Location> reachedLocations = new HashSet<Location>();
        
        for (Iterator it = outgoingSwitches.iterator(); it.hasNext();) {
            Switch s = (Switch) it.next();
            sb.append(name + " -> " + s.getTarget().getName() + ' ');
            sb.append("[label = \"");
            sb.append(s.toStringWithNewLines());
            sb.append("\"];\n");
            reachedLocations.add(s.getTarget());
        }
        
        return reachedLocations;
    }
}
