/* Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package info.frantzen.testing.ssmsimulator.ssm;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * A message is very close to what is a message in WSDL.
 * A message is always related to an operation.
 * A message is uniquely determined by its kind, its operation and its port.
 * We called a message in the FATES 04 paper a <I>gate</I>.
 * @author Lars Frantzen
 * @version 0.1
 */
public class Message {
    
    private String name;
    private MessageKind kind;
    private Operation op;
    private Port port;
    private ArrayList<InteractionVariable> type;
    
    /**
     * The reason to model quiescence internally as a message is that we can see quiescence then as a special instantiated message to
     * add it to a suspension trace.
     *
     * This method cannot be used by a user of the simulator, only by the simulator itself. Such a quiescence message
     * must not be part of the message-set of an SSM, and it does not pass the consistency check. See the Port class.
     */
    protected static Message generateQuiescenceMessage(Port p) {
        return new Message("quiescence", MessageKind.OUTPUT, null, p, null);
    }
    
    public static Message generateUnobservableMessageForPort(Port p) {
        return new Message("", MessageKind.UNOBSERVABLE, null, p, null);
    }
    
    /**
     * Create a new instance of Message.
     * @param name the name of the message
     * @param kind the kind of the message
     * @param op the Operation to which this message belongs
     * @param port the port to which this message belongs
     * @param type the type of this message, i.e. a list of interaction variables
     */
    public Message(String name, MessageKind kind, Operation op, Port port, ArrayList<InteractionVariable> type) {
        this.name = name;
        this.kind = kind;
        this.op = op;
        this.port = port;
        this.type = type;
    }
    
    /**
     * Getter for the name attribute.
     * @return the name of the Message
     */
    public String getName() {
        return name;
    }
    
    /**
     * Setter for the name attribute.
     * @param name the new name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Getter for the kind attribute.
     * @return the kind of the Message
     */
    public MessageKind getKind() {
        return kind;
    }
    
    /**
     * Setter of the kind attribute.
     * @param type the new kind of the Message
     */
    public void setKind(MessageKind type) {
        this.kind = type;
    }
    
    /**
     * Getter for the Operation attribute.
     * @return the Operation to which this Message belongs
     */
    public Operation getOperation() {
        return op;
    }
    
    /**
     * Getter of the Port attribute.
     * @return the port to which this Message belongs
     */
    public Port getPort() {
        return port;
    }
    
    /**
     * Getter for the type attribute.
     * @return the type of this Message
     */
    public ArrayList<InteractionVariable> getType() {
        return type;
    }
    
    /**
     * A String representation of this Message.
     * @return a String representation of this Message
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
//        sb.append(port.getName());
//        sb.append(":");
        
        if (op != null)
            sb.append(op.getName());
        
        if (kind == MessageKind.INPUT)
            sb.append("?");
        else if (kind == MessageKind.OUTPUT)
            sb.append("!");
        else
            sb.append("(unobservable)");
        
        //sb.append(name);
        return sb.toString();
    }
    
    public String toTypedString() {
        StringBuffer sb = new StringBuffer();
        sb.append(toString());
        
        if (type != null) {
            sb.append('<');
            for (Iterator it = type.iterator(); it.hasNext() ;) {
                InteractionVariable v = (InteractionVariable) it.next();
                sb.append(v.getName());
                sb.append(':');
                sb.append(v.getType());
                if (it.hasNext())
                    sb.append(',');
            }
            sb.append('>');
        }
        return sb.toString();
    }
    
    /**
     * Checks the consistency. Either the message is an observable message, i.e.<br>
     * <CODE>(name != "") && (kind != null) && (op != null) && (port != null) && (type != null)</CODE><br>
     * or the message is unobservable, i.e.<br>
     * <CODE>(name == "") && (kind == MessageKind.UNOBSERVABLE) && (op == null) && (port != null) && (type == null)</CODE>
     * @return true if the consistency is ok, false otherwise
     */
    public boolean checkConsistency() {
        return ( ((name != "") && (kind != null) && (op != null) && (port != null) && (type != null)) ||
                ((name == "") && (kind == MessageKind.UNOBSERVABLE) && (op == null) && (port != null) && (type == null))
                );
    }
    
}
