/* Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 */

package info.frantzen.testing.ssmsimulator.ssm;

import info.frantzen.testing.ssmsimulator.interpreter.MyNode;
import info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParser;
import info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation;
import info.frantzen.testing.ssmsimulator.types.ST_String;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

/**
 * A switch is a symbolic transition, i.e. a transition connecting two {@link Location locations}.<br>
 * <br>
 * Every switch consists of:<br>
 * - a target {@link Location location}<br>
 * - a {@link Message message}<br>
 * - a string for the syntax of the switch restriction<br>
 * - a string for the syntax of the update mapping<br>
 * - a {@link DumontParser parser} for the switch restriction<br>
 * - a {@link DumontParser parser} for the update mapping
 * @author Lars Frantzen
 * @version 0.1
 */
public class Switch {
    
    private Location target;
    private Message message;
    private String switchRestriction;
    private String updateMapping;
    private DumontParser srParser;
    private DumontParser umParser;
    private Set<Hashtable<String,Integer>> foundSolutions = new HashSet<Hashtable<String,Integer>>();
    private int visitedCount = 0;
    private boolean alwaysNewInputs = false;
    private boolean alwaysNewDeleteSolutions = false;
    
    /**
     * Creates a new switch.
     * @param target the target {@link Location location}
     * @param message the {@link Message message} sent or received via this switch
     * @param switchRestriction the syntax of the switch restriction
     * @param updateMapping the syntax of the update mapping
     */
    public Switch(Location target, Message message, String switchRestriction, String updateMapping) {
        this.target = target;
        this.message = message;
        this.switchRestriction = switchRestriction;
        this.updateMapping = updateMapping;
    }
    
    /**
     * Creates a new switch without fixing the target location.
     * @param message the {@link Message message} sent or received via this switch
     * @param switchRestriction the syntax of the switch restriction
     * @param updateMapping the syntax of the update mapping
     */
    public Switch(Message message, String switchRestriction, String updateMapping) {
        this.target = null;
        this.message = message;
        this.switchRestriction = switchRestriction;
        this.updateMapping = updateMapping;
    }
    
    public void setAlwaysNewInputs(boolean b) {
        alwaysNewInputs = b;
    }
    
    public boolean getAlwaysNewInputs() {
        return alwaysNewInputs;
    }
    
    public void setAlwaysNewDeleteSolutions(boolean b) {
        alwaysNewDeleteSolutions = b;
    }
    
    public boolean getAlwaysNewDeleteSolutions() {
        return alwaysNewDeleteSolutions;
    }
    
    public void visited() {
        visitedCount++;
    }
    
    public int getVistedCount() {
        return visitedCount;
    }
    
    /**
     * Getter for the message.
     * @return the message sent or received via this switch
     */
    public Message getMessage() {
        return message;
    }
    
    /**
     * Setter for the message.
     * @param message the new message sent or reveived via this switch
     */
    public void setMessage(Message message) {
        this.message = message;
    }
    
    /**
     * Returns the string representation of the switch restriction.
     * @return the string representation of the switch restriction
     */
    public String getSwitchRestriction() {
        return switchRestriction;
    }
    
    /**
     * Setter for the string representation of the switch restriction.
     * @param switchRestriction the new string representation of the switch restriction
     */
    public void setSwitchRestriction(String switchRestriction) {
        this.switchRestriction = switchRestriction;
    }
    
    /**
     * Getter for the target location.
     * @return the target location
     */
    public Location getTarget() {
        return target;
    }
    
    /**
     * Setter for the target location.
     * @param target the new target location
     */
    public void setTarget(Location target) {
        this.target = target;
    }
    
    /**
     * Returns the string representation of the update mapping.
     * @return the string representation of the update mapping
     */
    public String getUpdateMapping() {
        return updateMapping;
    }
    
    /**
     * Setter for the string representation of the update mapping.
     * @param updateMapping the new string representation of the update mapping
     */
    public void setUpdateMapping(String updateMapping) {
        this.updateMapping = updateMapping;
    }
    
    /**
     * A string representation of the switch.
     * @return s string representation of the switch
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(message.toString() + " [" + switchRestriction + "] " + updateMapping);
        return sb.toString();
    }
    
    public String toStringWithNewLines() {
        StringBuffer sb = new StringBuffer();
        String sr = switchRestriction;
        sr = sr.replaceAll("&&","&&\\\\n");
        sr = sr.replaceAll("\\|\\|","||\\\\n");
        sb.append(message.toTypedString() + "\\n[" + sr + "]\\n" + updateMapping);
        return sb.toString();
    }
    
    
    /**
     * Checks the consistency, i.e.<br>
     * <CODE>(target != null) && (message != null)</CODE>
     * @return true if the consistency is ok, false otherwise
     */
    public boolean checkConsistency() {
        return ((target != null) && (message != null));
    }
    
    /**
     * Attaches a parser for the switch restriction to the switch.
     * @param parser a parser for the switch restriction
     */
    public void attachSRParser(DumontParser parser) {
        this.srParser = parser;
    }
    
    /**
     * Attaches a parser for the update mapping to the switch.
     * @param parser a parser for the update mapping
     */
    public void attachUMParser(DumontParser parser) {
        this.umParser = parser;
    }
    
    /**
     * Calling this methods interprets the update mapping, i.e. the values of the location variables are
     * updated according to the update mapping.
     * @throws ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation While interpreting a runtime violation has been detected.
     */
    public void executeUpdateMapping() throws RuntimeViolation {
        umParser.interpretRootNode();
        if (MyNode.top == 0)
            MyNode.top--;
    }
    
    /**
     * Calling this methods interprets the switch restriction and the result (true or false) is
     * returned.
     * @return true if the switch restriction holds, false otherwise
     * @throws ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation While interpreting a runtime violation has been detected.
     */
    public boolean evaluateSwitchRestriction() throws RuntimeViolation {
        srParser.interpretRootNode();
        return ((Boolean) MyNode.stack[MyNode.top--]);
    }
    
    public String generatePrologQuery() throws RuntimeViolation {
        MyNode.prologVartab.clear();
        MyNode.prologVarCounter = 0;
        ST_String.resetInstanceNumberHandling();
        
        String pq = srParser.generatePrologQuery();
        return pq;
    }
    
    public void addSolution(Hashtable<String,Integer> sol) {
        foundSolutions.add(sol);
    }
    
    public Set<Hashtable<String,Integer>> getFoundSolutions() {
        return foundSolutions;
    }
    
    public void clearFoundSolutions() {
        foundSolutions.clear();
    }
    
    
}
